import logging
from telegram import Update
from telegram.ext import ContextTypes
from .base import InfoCommandsBase
from src.utils.token_display_formatter import get_formatter

logger = logging.getLogger(__name__)

class BalanceAdjustmentsCommands(InfoCommandsBase):
    """Handles all balance adjustment-related commands."""

    async def balance_adjustments_command(self, update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
        """Handle the /balance_adjustments command to show balance adjustment history."""
        if not self._is_authorized(update):
            return

        try:
            stats = self.trading_engine.get_stats()
            if not stats:
                await self._reply(update, "❌ Could not load trading statistics")
                return

            # Get balance adjustments
            adjustments = stats.get_balance_adjustments()
            if not adjustments:
                await self._reply(update, "📊 <b>Balance Adjustments</b>\n\n📭 No balance adjustments found.")
                return

            formatter = get_formatter()
            adjustments_text = ["📊 <b>Balance Adjustments History</b>"]

            # Sort adjustments by timestamp (newest first)
            sorted_adjustments = sorted(adjustments, key=lambda x: x.get('timestamp', ''), reverse=True)

            for adj in sorted_adjustments:
                amount = float(adj.get('amount', 0))
                reason = adj.get('reason', 'Unknown')
                timestamp = adj.get('timestamp', '')
                
                # Format the adjustment
                amount_emoji = "➕" if amount > 0 else "➖"
                amount_str = await formatter.format_price_with_symbol(abs(amount))
                
                adjustments_text.append(
                    f"\n{amount_emoji} <b>{amount_str}</b>"
                    f"\n• Reason: {reason}"
                    f"\n• Time: {timestamp}"
                )

            # Add summary
            total_adjustments = sum(float(adj.get('amount', 0)) for adj in adjustments)
            total_emoji = "🟢" if total_adjustments >= 0 else "🔴"
            total_str = await formatter.format_price_with_symbol(total_adjustments)
            
            adjustments_text.append(f"\n\n💼 <b>Total Adjustments:</b> {total_emoji} {total_str}")

            await self._reply(update, "\n".join(adjustments_text))

        except Exception as e:
            error_message = f"❌ Error processing balance adjustments command: {str(e)}"
            await self._reply(update, error_message)
            logger.error(f"Error in balance adjustments command: {e}") 