import logging
from typing import Optional, Dict, Any
from telegram import Update, Message
from telegram.ext import ContextTypes

logger = logging.getLogger(__name__)

class InfoCommandsBase:
    """Base class for all info commands with shared functionality."""
    
    def __init__(self, trading_engine, notification_manager=None):
        """Initialize the base class with required dependencies."""
        self.trading_engine = trading_engine
        self.notification_manager = notification_manager
        self.logger = logging.getLogger(__name__)

    def _is_authorized(self, update: Update) -> bool:
        """Check if the user is authorized to use the command."""
        if not update or not update.effective_chat:
            return False
        
        from src.config.config import Config
        chat_id = update.effective_chat.id
        authorized = str(chat_id) == str(Config.TELEGRAM_CHAT_ID)
        
        if not authorized:
            logger.warning(f"Unauthorized access attempt by chat_id: {chat_id}")
        
        return authorized

    async def _reply(self, update: Update, text: str, **kwargs) -> Optional[Message]:
        """Common reply method for all commands."""
        # Ensure default parse_mode is set if not provided
        if 'parse_mode' not in kwargs:
            kwargs['parse_mode'] = 'HTML'
            
        try:
            if update.callback_query:
                await update.callback_query.answer()
                result = await update.callback_query.edit_message_text(
                    text=text,
                    **kwargs
                )
                if isinstance(result, Message):
                    return result
                return update.callback_query.message  # Fallback for True case
            
            elif update.message:
                return await update.message.reply_text(
                    text=text,
                    **kwargs
                )
        except Exception as e:
            self.logger.error(f"Error sending reply: {e}")
            # Try fallback reply method
            try:
                if update.message:
                    await update.message.reply_text(
                        text="❌ Error processing command. Please try again.",
                        parse_mode=kwargs.get('parse_mode', 'HTML')
                    )
            except Exception as e2:
                self.logger.error(f"Error sending fallback reply: {e2}")
        
        return None

    def _get_formatter(self):
        """Get the token display formatter."""
        from src.utils.token_display_formatter import get_formatter
        return get_formatter()

    def _normalize_token_case(self, token: str) -> str:
        """Normalize token case for consistent display."""
        if not token:
            return token
        return token.upper()  # Add your token case normalization logic here
