import logging
from typing import Dict, Any, List, Optional
from telegram import Update
from telegram.ext import ContextTypes
from .base import InfoCommandsBase
from .utils import normalize_token_case
from src.config.config import Config
from src.utils.token_display_formatter import get_formatter

logger = logging.getLogger(__name__)

class MarketCommands(InfoCommandsBase):
    """Handles all market-related commands."""

    def __init__(self, trading_engine, notification_manager):
        super().__init__(trading_engine, notification_manager)
        self.formatter = get_formatter()

    async def market_command(self, update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
        """Handle the /market command to show market information."""
        
        # Send a loading message first
        sent_message = await self._reply(update, "⏳ Fetching market data...")
        if not sent_message:
            logger.error("Failed to send initial message in market_command.")
            return

        try:
            if not self._is_authorized(update):
                await context.bot.edit_message_text(chat_id=sent_message.chat.id, message_id=sent_message.message_id, text="❌ Unauthorized access.")
                return

            # Get token from command arguments or use default
            token = normalize_token_case(context.args[0]) if context.args else Config.DEFAULT_TRADING_TOKEN
            
            # Construct the full market symbol
            symbol = f"{token}/USDC:USDC"
            
            # Get market data
            market_data = await self.trading_engine.get_market_data(symbol)
            if not market_data:
                await context.bot.edit_message_text(chat_id=sent_message.chat.id, message_id=sent_message.message_id, text=f"❌ Could not get market data for {symbol}. Check if the symbol is correct.")
                return
            
            # Get all markets to find leverage info
            all_markets = await self.trading_engine.client.get_markets()
            market_info = all_markets.get(symbol) if all_markets else None
            
            # Extract ticker and orderbook for easier access
            ticker = market_data.get('ticker', {})
            orderbook = market_data.get('orderbook', {})
            
            # Safely get nested info dictionary
            ticker_info = ticker.get('info', {})

            # Calculate 24h change
            last_price = ticker.get('last', 0)
            prev_close = ticker.get('previousClose', 0)
            change_24h = ((last_price - prev_close) / prev_close * 100) if prev_close else 0
            
            # Calculate price position within the daily range
            high_price = ticker.get('high', 0) or 0
            low_price = ticker.get('low', 0) or 0
            
            percent_from_low_str = ""
            percent_from_high_str = ""
            if high_price > low_price and low_price > 0 and last_price > 0:
                total_range = high_price - low_price
                dist_from_low = last_price - low_price
                percent_from_low = (dist_from_low / total_range) * 100
                percent_from_high = 100 - percent_from_low
                percent_from_low_str = f" ({percent_from_low:.0f}% above)"
                percent_from_high_str = f" ({percent_from_high:.0f}% below)"

            # Format the message
            message = f"📊 <b>Market Information for {token}</b>\n\n"
            
            # Add price information
            message += "💰 <b>Price Information:</b>\n"
            message += f"Last Price: {await self.formatter.format_price_with_symbol(last_price)}\n"
            message += f"24h High: {await self.formatter.format_price_with_symbol(high_price)}{percent_from_high_str}\n"
            message += f"24h Low: {await self.formatter.format_price_with_symbol(low_price)}{percent_from_low_str}\n"
            message += f"24h Change: {change_24h:.2f}%\n"
            
            # Add leverage information if available
            if market_info:
                max_leverage = ticker_info.get('maxLeverage', 'N/A')
                if max_leverage != 'N/A':
                    message += f"Leverage: Up to {max_leverage}x\n"
            
            # Add volume information
            message += f"\n📈 <b>Volume Information:</b>\n"
            message += f"24h Volume: {await self.formatter.format_amount(float(ticker_info.get('dayBaseVlm', 0)), token)}\n"
            message += f"24h Turnover: {await self.formatter.format_price_with_symbol(float(ticker_info.get('dayNtlVlm', 0)))}\n"
            
            # Add order book information if available
            if orderbook:
                bid = ticker.get('bid', 0)
                ask = ticker.get('ask', 0)
                spread = ask - bid if ask and bid else 0
                message += f"\n📚 <b>Order Book:</b>\n"
                message += f"Bid: {await self.formatter.format_price_with_symbol(bid)}\n"
                message += f"Ask: {await self.formatter.format_price_with_symbol(ask)}\n"
                message += f"Spread: {await self.formatter.format_price_with_symbol(spread)}\n"
            
            await context.bot.edit_message_text(chat_id=sent_message.chat.id, message_id=sent_message.message_id, text=message.strip(), parse_mode='HTML')
            
        except Exception as e:
            logger.error(f"Error in market command: {e}")
            if sent_message:
                await context.bot.edit_message_text(chat_id=sent_message.chat.id, message_id=sent_message.message_id, text="❌ Error getting market information. Please try again later.")
