import logging
from telegram import Update
from telegram.ext import ContextTypes
from .base import InfoCommandsBase
from src.utils.token_display_formatter import get_formatter

logger = logging.getLogger(__name__)

class MonthlyCommands(InfoCommandsBase):
    """Handles all monthly performance-related commands."""

    async def monthly_command(self, update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
        """Handle the /monthly command to show monthly performance stats."""
        if not self._is_authorized(update):
            return

        try:
            stats = self.trading_engine.get_stats()
            if not stats:
                await self._reply(update, "❌ Could not load trading statistics")
                return

            monthly_stats_list = stats.get_monthly_stats(12)  # Last 12 months
            formatter = get_formatter()

            if not monthly_stats_list:
                await self._reply(update, "📅 <b>Monthly Performance</b>\n📭 No monthly performance data available yet.\n💡 Monthly stats are calculated from completed trades. Start trading to see them!")
                return

            monthly_text_parts = [f"📅 <b>Monthly Performance (Last 12 Months)</b>"]

            total_pnl_all_months = 0
            total_trades_all_months = 0
            total_roe_all_months = 0
            trading_months_count = 0

            period_lines = []
            for month_stats_item in monthly_stats_list:
                if month_stats_item['has_trades']:
                    pnl_emoji = "🟢" if month_stats_item['pnl'] >= 0 else "🔴"
                    pnl_str = await formatter.format_price_with_symbol(month_stats_item['pnl'])
                    roe = month_stats_item.get('roe', 0.0)  # Get ROE from stats
                    roe_str = f"ROE: {roe:+.1f}%" if roe != 0 else ""
                    period_lines.append(f"📅 <b>{month_stats_item['month_formatted']}</b>: {pnl_emoji} {pnl_str} ({month_stats_item['pnl_pct']:+.1f}%) {roe_str} | Trades: {month_stats_item['trades']}")
                    total_pnl_all_months += month_stats_item['pnl']
                    total_trades_all_months += month_stats_item['trades']
                    total_roe_all_months += roe
                    trading_months_count += 1
                else:
                    period_lines.append(f"📅 <b>{month_stats_item['month_formatted']}</b>: 📭 No trading activity")

            if period_lines:
                monthly_text_parts.append("\n".join(period_lines))

            if trading_months_count > 0:
                avg_monthly_pnl = total_pnl_all_months / trading_months_count
                avg_monthly_roe = total_roe_all_months / trading_months_count
                avg_pnl_emoji = "🟢" if avg_monthly_pnl >= 0 else "🔴"
                total_pnl_all_months_str = await formatter.format_price_with_symbol(total_pnl_all_months)
                avg_monthly_pnl_str = await formatter.format_price_with_symbol(avg_monthly_pnl)

                monthly_text_parts.append(f"\n\n📈 <b>Period Summary:</b>")
                monthly_text_parts.append(f"  Total P&L: {avg_pnl_emoji} {total_pnl_all_months_str} | Avg Monthly: {avg_monthly_pnl_str}")
                monthly_text_parts.append(f"  Total ROE: {total_roe_all_months:+.1f}% | Avg Monthly ROE: {avg_monthly_roe:+.1f}%")
                monthly_text_parts.append(f"  Trading Months: {trading_months_count}/12 | Total Trades: {total_trades_all_months}")
            else:
                if not period_lines:
                    monthly_text_parts = [monthly_text_parts[0]]
                monthly_text_parts.append("\n\n📉 No trading activity in the last 12 months.")

            await self._reply(update, "\n".join(monthly_text_parts).strip())

        except Exception as e:
            error_message = f"❌ Error processing monthly command: {str(e)}"
            await self._reply(update, error_message)
            logger.error(f"Error in monthly command: {e}") 