import logging
from telegram import Update
from telegram.ext import ContextTypes
from .base import InfoCommandsBase
from src.utils.token_display_formatter import get_formatter

logger = logging.getLogger(__name__)

class WeeklyCommands(InfoCommandsBase):
    """Handles all weekly performance-related commands."""

    async def weekly_command(self, update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
        """Handle the /weekly command to show weekly performance stats."""
        if not self._is_authorized(update):
            return

        try:
            stats = self.trading_engine.get_stats()
            if not stats:
                await self._reply(update, "❌ Could not load trading statistics")
                return

            weekly_stats_list = stats.get_weekly_stats(10)  # Last 10 weeks
            formatter = get_formatter()

            if not weekly_stats_list:
                await self._reply(update, "📅 <b>Weekly Performance</b>\n📭 No weekly performance data available yet.\n💡 Weekly stats are calculated from completed trades. Start trading to see them!")
                return

            weekly_text_parts = [f"📅 <b>Weekly Performance (Last 10 Weeks)</b>"]

            total_pnl_all_weeks = 0
            total_trades_all_weeks = 0
            total_roe_all_weeks = 0
            trading_weeks_count = 0

            period_lines = []
            for week_stats_item in weekly_stats_list:
                if week_stats_item['has_trades']:
                    pnl_emoji = "🟢" if week_stats_item['pnl'] >= 0 else "🔴"
                    pnl_str = await formatter.format_price_with_symbol(week_stats_item['pnl'])
                    roe = week_stats_item.get('roe', 0.0)  # Get ROE from stats
                    roe_str = f"ROE: {roe:+.1f}%" if roe != 0 else ""
                    period_lines.append(f"📅 <b>{week_stats_item['week_formatted']}</b>: {pnl_emoji} {pnl_str} ({week_stats_item['pnl_pct']:+.1f}%) {roe_str} | Trades: {week_stats_item['trades']}")
                    total_pnl_all_weeks += week_stats_item['pnl']
                    total_trades_all_weeks += week_stats_item['trades']
                    total_roe_all_weeks += roe
                    trading_weeks_count += 1
                else:
                    period_lines.append(f"📅 <b>{week_stats_item['week_formatted']}</b>: 📭 No trading activity")

            if period_lines:
                weekly_text_parts.append("\n".join(period_lines))

            if trading_weeks_count > 0:
                avg_weekly_pnl = total_pnl_all_weeks / trading_weeks_count
                avg_weekly_roe = total_roe_all_weeks / trading_weeks_count
                avg_pnl_emoji = "🟢" if avg_weekly_pnl >= 0 else "🔴"
                total_pnl_all_weeks_str = await formatter.format_price_with_symbol(total_pnl_all_weeks)
                avg_weekly_pnl_str = await formatter.format_price_with_symbol(avg_weekly_pnl)

                weekly_text_parts.append(f"\n\n📈 <b>Period Summary:</b>")
                weekly_text_parts.append(f"  Total P&L: {avg_pnl_emoji} {total_pnl_all_weeks_str} | Avg Weekly: {avg_weekly_pnl_str}")
                weekly_text_parts.append(f"  Total ROE: {total_roe_all_weeks:+.1f}% | Avg Weekly ROE: {avg_weekly_roe:+.1f}%")
                weekly_text_parts.append(f"  Trading Weeks: {trading_weeks_count}/10 | Total Trades: {total_trades_all_weeks}")
            else:
                if not period_lines:
                    weekly_text_parts = [weekly_text_parts[0]]
                weekly_text_parts.append("\n\n📉 No trading activity in the last 10 weeks.")

            await self._reply(update, "\n".join(weekly_text_parts).strip())

        except Exception as e:
            error_message = f"❌ Error processing weekly command: {str(e)}"
            await self._reply(update, error_message)
            logger.error(f"Error in weekly command: {e}") 