import os
from dotenv import load_dotenv
from typing import Optional
import logging

# Load environment variables from .env file
load_dotenv()

logger = logging.getLogger(__name__)

def get_bool_env(var_name: str, default: str = 'false') -> bool:
    """Gets a boolean environment variable."""
    return os.getenv(var_name, default,).lower() in ('true', '1', 't')

class Config:
    """Configuration class for the Hyperliquid trading bot."""
    
    # Hyperliquid API Configuration
    HYPERLIQUID_SECRET_KEY: Optional[str] = os.getenv('HYPERLIQUID_SECRET_KEY')    # API generator key
    HYPERLIQUID_WALLET_ADDRESS: Optional[str] = os.getenv('HYPERLIQUID_WALLET_ADDRESS')  # Wallet address
    HYPERLIQUID_TESTNET: bool = get_bool_env('HYPERLIQUID_TESTNET', 'true')
    HYPERLIQUID_MARGIN_MODE = os.getenv('HYPERLIQUID_MARGIN_MODE', 'ISOLATED').upper()
    
    # Trading Bot Configuration
    DEFAULT_TRADING_TOKEN: str = os.getenv('DEFAULT_TRADING_TOKEN', 'BTC')
    RISK_MANAGEMENT_ENABLED: bool = get_bool_env('RISK_MANAGEMENT_ENABLED', 'true')
    STOP_LOSS_PERCENTAGE: float = float(os.getenv('STOP_LOSS_PERCENTAGE', '10.0'))
    
    # RSI Notification Configuration
    RSI_NOTIFICATION_ENABLED: bool = get_bool_env('RSI_NOTIFICATION_ENABLED', 'true')
    RSI_TIMEFRAME: str = os.getenv('RSI_TIMEFRAME', '1h')
    RSI_PERIOD: int = int(os.getenv('RSI_PERIOD', '14'))
    RSI_SMA_PERIOD: int = int(os.getenv('RSI_SMA_PERIOD', '5'))
    
    # Copy Trading Configuration
    COPY_TRADING_ENABLED: bool = get_bool_env('COPY_TRADING_ENABLED', 'false')
    COPY_TRADING_TARGET_ADDRESS: Optional[str] = os.getenv('COPY_TRADING_TARGET_ADDRESS')
    COPY_TRADING_PORTFOLIO_PERCENTAGE: float = float(os.getenv('COPY_TRADING_PORTFOLIO_PERCENTAGE', '0.1'))
    COPY_TRADING_MODE: str = os.getenv('COPY_TRADING_MODE', 'PROPORTIONAL').upper()
    COPY_TRADING_MAX_LEVERAGE: float = float(os.getenv('COPY_TRADING_MAX_LEVERAGE', '10.0'))
    COPY_TRADING_MIN_POSITION_SIZE: float = float(os.getenv('COPY_TRADING_MIN_POSITION_SIZE', '10.0'))
    COPY_TRADING_EXECUTION_DELAY: float = float(os.getenv('COPY_TRADING_EXECUTION_DELAY', '5.0'))
    COPY_TRADING_NOTIFICATIONS: bool = get_bool_env('COPY_TRADING_NOTIFICATIONS', 'true')
    COPY_TRADING_ONLY_NEW_TRADES: bool = get_bool_env('COPY_TRADING_ONLY_NEW_TRADES', 'true')
    
    # Telegram Bot Configuration
    TELEGRAM_BOT_TOKEN: Optional[str] = os.getenv('TELEGRAM_BOT_TOKEN')
    TELEGRAM_CHAT_ID: Optional[str] = os.getenv('TELEGRAM_CHAT_ID')
    TELEGRAM_ENABLED: bool = get_bool_env('TELEGRAM_ENABLED', 'true')
    TELEGRAM_DROP_PENDING_UPDATES: bool = get_bool_env('TELEGRAM_DROP_PENDING_UPDATES', 'true')
    
    # Custom Keyboard Configuration
    TELEGRAM_CUSTOM_KEYBOARD_ENABLED: bool = get_bool_env('TELEGRAM_CUSTOM_KEYBOARD_ENABLED', 'true')
    TELEGRAM_CUSTOM_KEYBOARD_LAYOUT: str = os.getenv('TELEGRAM_CUSTOM_KEYBOARD_LAYOUT', '/daily,/performance,/balance|/stats,/positions,/orders|/price,/market,/help,/commands')
    
    # Bot settings
    BOT_HEARTBEAT_SECONDS: int = int(os.getenv('BOT_HEARTBEAT_SECONDS', '5'))
    
    # Removed market monitor settings - simplified architecture
    
    # Order settings
    DEFAULT_SLIPPAGE: float = 0.005  # 0.5%
    
    # Logging
    LOG_LEVEL: str = os.getenv('LOG_LEVEL', 'INFO').upper()
    
    # Log file configuration
    LOG_TO_FILE: bool = get_bool_env('LOG_TO_FILE', 'true')
    LOG_FILE_PATH: str = os.getenv('LOG_FILE_PATH', 'logs/trading_bot.log')
    LOG_MAX_SIZE_MB: int = int(os.getenv('LOG_MAX_SIZE_MB', '10'))
    LOG_BACKUP_COUNT: int = int(os.getenv('LOG_BACKUP_COUNT', '5'))
    LOG_ROTATION_TYPE: str = os.getenv('LOG_ROTATION_TYPE', 'size')  # 'size' or 'time'
    LOG_ROTATION_INTERVAL: str = os.getenv('LOG_ROTATION_INTERVAL', 'midnight')  # for time rotation
    
    @classmethod
    def validate(cls) -> bool:
        """Validate all configuration settings."""
        validators = [
            cls._validate_hyperliquid,
            cls._validate_telegram,
            cls._validate_bot_settings,
            cls._validate_rsi,
            cls._validate_copy_trading,
            cls._validate_logging
        ]
        return all(validator() for validator in validators)

    @classmethod
    def _validate_hyperliquid(cls) -> bool:
        """Validate Hyperliquid settings."""
        if not cls.HYPERLIQUID_WALLET_ADDRESS:
            logger.error("❌ HYPERLIQUID_WALLET_ADDRESS is required")
            return False
        if not cls.HYPERLIQUID_SECRET_KEY:
            logger.error("❌ HYPERLIQUID_SECRET_KEY (API generator key) is required")
            return False
        
        # Validate margin mode
        valid_margin_modes = ['ISOLATED', 'CROSS']
        if cls.HYPERLIQUID_MARGIN_MODE not in valid_margin_modes:
            logger.error(f"❌ HYPERLIQUID_MARGIN_MODE '{cls.HYPERLIQUID_MARGIN_MODE}' is not valid. Valid options: {valid_margin_modes}")
            return False
            
        return True

    @classmethod
    def _validate_telegram(cls) -> bool:
        """Validate Telegram settings."""
        if cls.TELEGRAM_ENABLED:
            if not cls.TELEGRAM_BOT_TOKEN:
                logger.error("❌ TELEGRAM_BOT_TOKEN is required when Telegram is enabled")
                return False
            if not cls.TELEGRAM_CHAT_ID:
                logger.error("❌ TELEGRAM_CHAT_ID is required when Telegram is enabled")
                return False
        return True

    @classmethod
    def _validate_bot_settings(cls) -> bool:
        """Validate general bot settings."""
        if cls.BOT_HEARTBEAT_SECONDS < 5:
            logger.error("❌ BOT_HEARTBEAT_SECONDS must be at least 5 seconds to avoid rate limiting")
            return False
        if cls.BOT_HEARTBEAT_SECONDS > 600:
            logger.warning("⚠️ BOT_HEARTBEAT_SECONDS is very high (>10 minutes), monitoring may be slow")
        if not cls.DEFAULT_TRADING_TOKEN:
            logger.error("❌ DEFAULT_TRADING_TOKEN must be set")
            return False
        return True

    @classmethod
    def _validate_rsi(cls) -> bool:
        """Validate RSI notification configuration."""
        valid_timeframes = ['1m', '3m', '5m', '15m', '30m', '1h', '2h', '4h', '6h', '8h', '12h', '1d', '3d', '1w', '1M']
        
        if cls.RSI_TIMEFRAME not in valid_timeframes:
            logger.error(f"❌ RSI_TIMEFRAME '{cls.RSI_TIMEFRAME}' is not valid. Valid: {valid_timeframes}")
            return False
        
        if cls.RSI_PERIOD < 2:
            logger.error("❌ RSI_PERIOD must be at least 2")
            return False
            
        if cls.RSI_SMA_PERIOD < 2:
            logger.error("❌ RSI_SMA_PERIOD must be at least 2")
            return False
        
        return True

    @classmethod
    def _validate_copy_trading(cls) -> bool:
        """Validate copy trading configuration."""
        if cls.COPY_TRADING_ENABLED:
            if not cls.COPY_TRADING_TARGET_ADDRESS:
                logger.warning("⚠️ Copy trading enabled but no target address set")
            elif not cls.COPY_TRADING_TARGET_ADDRESS.startswith('0x') or len(cls.COPY_TRADING_TARGET_ADDRESS) != 42:
                logger.error("❌ COPY_TRADING_TARGET_ADDRESS must be a valid Ethereum address (0x...)")
                return False
        
        if not (0.01 <= cls.COPY_TRADING_PORTFOLIO_PERCENTAGE <= 1.0):
            logger.error("❌ COPY_TRADING_PORTFOLIO_PERCENTAGE must be between 0.01 (1%) and 1.0 (100%)")
            return False
        
        if cls.COPY_TRADING_MODE not in ['PROPORTIONAL', 'FIXED']:
            logger.error("❌ COPY_TRADING_MODE must be 'PROPORTIONAL' or 'FIXED'")
            return False
        
        if not (1.0 <= cls.COPY_TRADING_MAX_LEVERAGE <= 50.0):
            logger.error("❌ COPY_TRADING_MAX_LEVERAGE must be between 1.0 and 50.0")
            return False
        
        if cls.COPY_TRADING_MIN_POSITION_SIZE < 1.0:
            logger.error("❌ COPY_TRADING_MIN_POSITION_SIZE must be at least $1.0")
            return False
        
        if cls.COPY_TRADING_EXECUTION_DELAY < 0:
            logger.error("❌ COPY_TRADING_EXECUTION_DELAY must be non-negative")
            return False
        
        return True

    @classmethod
    def _validate_logging(cls) -> bool:
        """Validate logging settings."""
        if cls.LOG_MAX_SIZE_MB <= 0:
            logger.error("❌ LOG_MAX_SIZE_MB must be positive")
            return False
        if cls.LOG_BACKUP_COUNT < 0:
            logger.error("❌ LOG_BACKUP_COUNT must be non-negative")
            return False
        if cls.LOG_ROTATION_TYPE not in ['size', 'time']:
            logger.error("❌ LOG_ROTATION_TYPE must be 'size' or 'time'")
            return False
        return True
    
    @classmethod
    def get_hyperliquid_config(cls) -> dict:
        """Get Hyperliquid configuration in CCXT format."""
        config = {
            'testnet': cls.HYPERLIQUID_TESTNET,
            'sandbox': cls.HYPERLIQUID_TESTNET,  # CCXT uses sandbox for testnet
        }
        
        # Add authentication if available
        # Hyperliquid CCXT implementation expects:
        # - apiKey: API generator key (SECRET_KEY)  
        # - walletAddress: wallet address (WALLET_ADDRESS)
        # - secret: (optional, set to wallet address for compatibility)
        if cls.HYPERLIQUID_SECRET_KEY:
            config['apiKey'] = cls.HYPERLIQUID_SECRET_KEY  # API generator key
            
        if cls.HYPERLIQUID_WALLET_ADDRESS:
            config['walletAddress'] = cls.HYPERLIQUID_WALLET_ADDRESS  # Wallet address
            config['secret'] = cls.HYPERLIQUID_WALLET_ADDRESS  # Also set as secret for backward compatibility
            
        return config
    
    @classmethod
    def print_config(cls):
        """Print current configuration (hiding sensitive data)."""
        logger.info("🔧 Configuration:")
        logger.info(f"  📡 Hyperliquid Testnet: {cls.HYPERLIQUID_TESTNET}")
        logger.info(f"  🏠 Wallet Address: {'✅ Set' if cls.HYPERLIQUID_WALLET_ADDRESS else '❌ Missing'}")
        logger.info(f"  🔑 API Generator Key: {'✅ Set' if cls.HYPERLIQUID_SECRET_KEY else '❌ Missing'}")
        logger.info(f"  ⚖️ Margin Mode: {cls.HYPERLIQUID_MARGIN_MODE}")
        logger.info(f"  📱 Telegram: {'✅ Enabled' if cls.TELEGRAM_ENABLED else '❌ Disabled'}")
        logger.info(f"  🔍 Log Level: {cls.LOG_LEVEL}")
        logger.info(f"  💾 Log to File: {cls.LOG_TO_FILE}")
        logger.info(f"  ⏱️ Heartbeat: {cls.BOT_HEARTBEAT_SECONDS}s")
        logger.info(f"  💰 DEFAULT_TRADING_TOKEN: {cls.DEFAULT_TRADING_TOKEN}")
        logger.info(f"  🧮 RISK_MANAGEMENT_ENABLED: {cls.RISK_MANAGEMENT_ENABLED}")
        logger.info(f"  🔄 STOP_LOSS_PERCENTAGE: {cls.STOP_LOSS_PERCENTAGE}%")
        logger.info(f"  📊 RSI_NOTIFICATION_ENABLED: {cls.RSI_NOTIFICATION_ENABLED}")
        logger.info(f"  ⏰ RSI_TIMEFRAME: {cls.RSI_TIMEFRAME}")
        logger.info(f"  🔢 RSI_PERIOD: {cls.RSI_PERIOD}")
        logger.info(f"  📈 RSI_SMA_PERIOD: {cls.RSI_SMA_PERIOD}")
        logger.info(f"  🔄 COPY_TRADING_ENABLED: {cls.COPY_TRADING_ENABLED}")
        logger.info(f"  🎯 COPY_TRADING_TARGET: {'✅ Set' if cls.COPY_TRADING_TARGET_ADDRESS else '❌ Not Set'}")
        logger.info(f"  💰 COPY_TRADING_PORTFOLIO: {cls.COPY_TRADING_PORTFOLIO_PERCENTAGE:.1%}")
        logger.info(f"  📊 COPY_TRADING_MODE: {cls.COPY_TRADING_MODE}")
        logger.info(f"  ⚡ COPY_TRADING_MAX_LEVERAGE: {cls.COPY_TRADING_MAX_LEVERAGE}x")
        logger.info(f"  🤖 TELEGRAM_BOT_TOKEN: {'✅ Set' if cls.TELEGRAM_BOT_TOKEN else '❌ Not Set'}")
        logger.info(f"  💬 TELEGRAM_CHAT_ID: {'✅ Set' if cls.TELEGRAM_CHAT_ID else '❌ Not Set'}")
        logger.info(f"  ⌨️ CUSTOM_KEYBOARD: {'✅ Enabled' if cls.TELEGRAM_CUSTOM_KEYBOARD_ENABLED else '❌ Disabled'}")
        logger.info(f"  🔍 LOG_FILE_PATH: {cls.LOG_FILE_PATH}")
        logger.info(f"  🔄 LOG_MAX_SIZE_MB: {cls.LOG_MAX_SIZE_MB} MB")
        logger.info(f"  🔄 LOG_BACKUP_COUNT: {cls.LOG_BACKUP_COUNT}")
        logger.info(f"  🔄 LOG_ROTATION_TYPE: {cls.LOG_ROTATION_TYPE}")
        logger.info(f"  🔄 LOG_ROTATION_INTERVAL: {cls.LOG_ROTATION_INTERVAL}") 