#!/usr/bin/env python3
"""
Logging configuration module with rotation and cleanup support.
"""

import logging
import logging.handlers
from pathlib import Path
from datetime import datetime, timedelta
from typing import Optional
from src.config.config import Config


class ColorFormatter(logging.Formatter):
    """A logging formatter that adds color to log levels."""

    COLORS = {
        'WARNING': '\033[93m',  # Yellow
        'INFO': '\033[92m',     # Green
        'DEBUG': '\033[94m',    # Blue
        'CRITICAL': '\033[91m', # Red
        'ERROR': '\033[91m',    # Red
    }
    RESET = '\033[0m'

    def format(self, record):
        log_message = super().format(record)
        return f"{self.COLORS.get(record.levelname, '')}{log_message}{self.RESET}"


class LoggingManager:
    """Manages logging configuration with rotation and cleanup."""
    
    def __init__(self):
        """Initialize the logging manager."""
        self.logger = None
        self.file_handler = None
        self.console_handler = None
        
    def setup_logging(self) -> logging.Logger:
        """
        Set up comprehensive logging with rotation and cleanup.
        
        Returns:
            Configured logger instance
        """
        # Create logs directory if it doesn't exist
        if Config.LOG_TO_FILE:
            log_dir = Path(Config.LOG_FILE_PATH).parent
            log_dir.mkdir(parents=True, exist_ok=True)
        
        # Create root logger
        logger = logging.getLogger()
        logger.setLevel(getattr(logging, Config.LOG_LEVEL.upper()))
        
        # Clear any existing handlers
        logger.handlers.clear()
        
        # Create formatter
        formatter = logging.Formatter(
            fmt='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
            datefmt='%Y-%m-%d %H:%M:%S'
        )
        
        # Set up console handler with color
        self.console_handler = logging.StreamHandler()
        self.console_handler.setLevel(getattr(logging, Config.LOG_LEVEL.upper()))
        
        # Use color formatter for console, and standard for file
        if self.console_handler.stream.isatty():
            color_formatter = ColorFormatter(
                fmt='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
                datefmt='%Y-%m-%d %H:%M:%S'
            )
            self.console_handler.setFormatter(color_formatter)
        else:
            self.console_handler.setFormatter(formatter)
        
        logger.addHandler(self.console_handler)
        
        # Set up file handler with rotation (if enabled)
        if Config.LOG_TO_FILE:
            if Config.LOG_ROTATION_TYPE == 'size':
                self.file_handler = logging.handlers.RotatingFileHandler(
                    filename=Config.LOG_FILE_PATH,
                    maxBytes=Config.LOG_MAX_SIZE_MB * 1024 * 1024,  # Convert MB to bytes
                    backupCount=Config.LOG_BACKUP_COUNT,
                    encoding='utf-8'
                )
            else:  # time-based rotation
                self.file_handler = logging.handlers.TimedRotatingFileHandler(
                    filename=Config.LOG_FILE_PATH,
                    when=Config.LOG_ROTATION_INTERVAL,
                    backupCount=Config.LOG_BACKUP_COUNT,
                    encoding='utf-8'
                )
            
            self.file_handler.setLevel(getattr(logging, Config.LOG_LEVEL.upper()))
            self.file_handler.setFormatter(formatter)
            logger.addHandler(self.file_handler)
        
        # Store reference
        self.logger = logger
        
        # Log the configuration
        logger.info("🔧 Logging system initialized")
        logger.info(f"📊 Log Level: {Config.LOG_LEVEL}")
        logger.info(f"📁 Log to File: {Config.LOG_TO_FILE}")
        
        if Config.LOG_TO_FILE:
            logger.info(f"📂 Log File: {Config.LOG_FILE_PATH}")
            logger.info(f"🔄 Rotation Type: {Config.LOG_ROTATION_TYPE}")
            logger.info(f"📏 Max Size: {Config.LOG_MAX_SIZE_MB}MB (size rotation)")
            logger.info(f"🗃️ Backup Count: {Config.LOG_BACKUP_COUNT}")
            logger.info(f"⏰ Rotation Interval: {Config.LOG_ROTATION_INTERVAL} (time rotation)")
        
        return logger
    
    def cleanup_old_logs(self, days_to_keep: int = 30) -> None:
        """
        Clean up old log files beyond a specified number of days.
        
        Args:
            days_to_keep: Number of days worth of logs to keep.
        """
        if not Config.LOG_TO_FILE:
            return

        log_dir = Path(Config.LOG_FILE_PATH).parent
        if not log_dir.exists():
            return

        cutoff_date = datetime.now() - timedelta(days=days_to_keep)
        removed_count = 0

        for log_file in log_dir.glob(f"{Path(Config.LOG_FILE_PATH).stem}*"):
            if log_file.is_file():
                try:
                    file_mod_time = datetime.fromtimestamp(log_file.stat().st_mtime)
                    if file_mod_time < cutoff_date:
                        log_file.unlink()
                        removed_count += 1
                        if self.logger:
                            self.logger.info(f"🗑️ Removed old log file: {log_file.name}")
                except (OSError, FileNotFoundError) as e:
                    if self.logger:
                        self.logger.warning(f"⚠️ Could not remove {log_file.name}: {e}")
        
        if removed_count > 0 and self.logger:
            self.logger.info(f"🧹 Cleanup complete: removed {removed_count} old log files.")
            
    def get_log_stats(self) -> dict:
        """
        Get statistics about current log files.
        
        Returns:
            Dictionary with log file statistics.
        """
        if not Config.LOG_TO_FILE:
            return {
                'log_to_file': False,
                'log_file_path': 'N/A',
                'current_size_mb': 0,
                'backup_files': 0,
                'total_size_mb': 0
            }

        log_file = Path(Config.LOG_FILE_PATH)
        log_dir = log_file.parent
        
        stats = {
            'log_to_file': True,
            'log_file_path': str(log_file),
            'current_size_mb': 0,
            'backup_files': 0,
            'total_size_mb': 0
        }

        if not log_dir.exists():
            return stats

        try:
            current_size = log_file.stat().st_size if log_file.exists() else 0
            stats['current_size_mb'] = current_size / (1024 * 1024)
            
            total_size = current_size
            backup_files = 0
            
            for f in log_dir.glob(f"{log_file.stem}*"):
                if f.is_file() and f != log_file:
                    total_size += f.stat().st_size
                    backup_files += 1
            
            stats['total_size_mb'] = total_size / (1024 * 1024)
            stats['backup_files'] = backup_files

        except (OSError, FileNotFoundError) as e:
            if self.logger:
                self.logger.error(f"❌ Error getting log stats: {e}")
        
        return stats
    
    def format_log_stats(self, stats: Optional[dict] = None) -> str:
        """
        Format log statistics for display.
        
        Args:
            stats: Log statistics dictionary (will fetch if None)
            
        Returns:
            Formatted string with log statistics
        """
        if stats is None:
            stats = self.get_log_stats()
        
        if not stats['log_to_file']:
            return "📊 <b>Logging:</b> Console only"
        
        return f"""📊 <b>Log File Statistics:</b>
• File: {Path(stats['log_file_path']).name}
• Current Size: {stats['current_size_mb']:.2f} MB
• Backup Files: {stats['backup_files']}
• Total Size: {stats['total_size_mb']:.2f} MB
• Max Size: {Config.LOG_MAX_SIZE_MB} MB
• Rotation: {Config.LOG_ROTATION_TYPE}
• Backups Kept: {Config.LOG_BACKUP_COUNT}"""


# Global logging manager instance
logging_manager = LoggingManager()


def setup_logging() -> logging.Logger:
    """
    Set up logging configuration.
    
    Returns:
        Configured logger instance
    """
    return logging_manager.setup_logging()


def cleanup_logs(days_to_keep: int = 30) -> None:
    """
    Clean up old log files.
    
    Args:
        days_to_keep: Number of days worth of logs to keep
    """
    logging_manager.cleanup_old_logs(days_to_keep)


def get_log_stats() -> dict:
    """Get log file statistics."""
    return logging_manager.get_log_stats()


def format_log_stats(stats: Optional[dict] = None) -> str:
    """Format log statistics for display."""
    return logging_manager.format_log_stats(stats) 