import sqlite3
import os
import logging

# Configure logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

# Define the database path consistently. Assuming it's the same as in TradingStats.
# If TradingStats uses a different relative path, adjust this.
DB_PATH = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'data', 'trading_stats.sqlite') # This resolves to <project_root>/data/trading_stats.sqlite

# Define the expected schema for the 'trades' table
# This should be the single source of truth for the trades table columns
TRADES_TABLE_SCHEMA = {
    "id": "INTEGER PRIMARY KEY AUTOINCREMENT",
    "exchange_fill_id": "TEXT UNIQUE",
    "timestamp": "TEXT NOT NULL",
    "symbol": "TEXT NOT NULL",
    "side": "TEXT NOT NULL",
    "amount": "REAL NOT NULL",
    "price": "REAL NOT NULL",
    "value": "REAL NOT NULL",
    "trade_type": "TEXT NOT NULL",
    "pnl": "REAL DEFAULT 0.0", # This seems like an old PNL field, might be realized_pnl or needs review
    "linked_order_table_id": "INTEGER",
    "status": "TEXT DEFAULT 'executed'",
    "trade_lifecycle_id": "TEXT",
    "position_side": "TEXT",
    "entry_price": "REAL",
    "current_position_size": "REAL DEFAULT 0",
    "entry_order_id": "TEXT",
    "stop_loss_order_id": "TEXT",
    "take_profit_order_id": "TEXT",
    "stop_loss_price": "REAL",
    "take_profit_price": "REAL",
    "realized_pnl": "REAL DEFAULT 0",
    "unrealized_pnl": "REAL DEFAULT 0",
    "mark_price": "REAL DEFAULT 0",
    "position_value": "REAL DEFAULT NULL",
    "unrealized_pnl_percentage": "REAL DEFAULT NULL",
    "liquidation_price": "REAL DEFAULT NULL",
    "margin_used": "REAL DEFAULT NULL",
    "leverage": "REAL DEFAULT NULL",
    "position_opened_at": "TEXT",
    "position_closed_at": "TEXT",
    "updated_at": "TEXT DEFAULT CURRENT_TIMESTAMP", # Ensure this default is correctly applied by SQLite
    "notes": "TEXT",
    "roe_percentage": "REAL DEFAULT NULL"
}

TOKEN_STATS_TABLE_SCHEMA = {
    "token": "TEXT PRIMARY KEY",
    "total_realized_pnl": "REAL DEFAULT 0.0",
    "total_completed_cycles": "INTEGER DEFAULT 0",
    "winning_cycles": "INTEGER DEFAULT 0",
    "losing_cycles": "INTEGER DEFAULT 0",
    "total_entry_volume": "REAL DEFAULT 0.0",
    "total_exit_volume": "REAL DEFAULT 0.0",
    "sum_of_winning_pnl": "REAL DEFAULT 0.0",
    "sum_of_losing_pnl": "REAL DEFAULT 0.0",
    "largest_winning_cycle_pnl": "REAL DEFAULT 0.0",
    "largest_losing_cycle_pnl": "REAL DEFAULT 0.0",
    "largest_winning_cycle_entry_volume": "REAL DEFAULT 0.0",
    "largest_losing_cycle_entry_volume": "REAL DEFAULT 0.0",
    "first_cycle_closed_at": "TEXT",
    "last_cycle_closed_at": "TEXT",
    "total_cancelled_cycles": "INTEGER DEFAULT 0",
    "total_duration_seconds": "INTEGER DEFAULT 0",
    "updated_at": "TEXT DEFAULT CURRENT_TIMESTAMP",
    "total_roe_percentage": "REAL DEFAULT 0.0",
    "best_roe_pnl": "REAL DEFAULT 0.0",
    "best_roe_percentage": "REAL DEFAULT 0.0",
    "worst_roe_pnl": "REAL DEFAULT 0.0",
    "worst_roe_percentage": "REAL DEFAULT 0.0"
}

def get_existing_columns(conn: sqlite3.Connection, table_name: str) -> list[str]:
    """Fetches the list of existing column names for a given table."""
    cursor = conn.cursor()
    cursor.execute(f"PRAGMA table_info({table_name});")
    return [row[1] for row in cursor.fetchall()]

def add_missing_columns(conn: sqlite3.Connection, table_name: str, expected_schema: dict[str, str]):
    """Adds missing columns to the table based on the expected schema."""
    existing_columns = get_existing_columns(conn, table_name)
    cursor = conn.cursor()
    added_columns = False

    for column_name, column_definition in expected_schema.items():
        if column_name not in existing_columns:
            try:
                # Basic ALTER TABLE statement. For defaults, SQLite adds them correctly for new rows.
                # Existing rows will have NULL for columns added this way unless a specific default is part of ADD COLUMN.
                # For simple REAL DEFAULT NULL or TEXT DEFAULT NULL, this is fine.
                # For REAL DEFAULT 0, SQLite handles this correctly.
                # For TEXT DEFAULT CURRENT_TIMESTAMP, this default applies on INSERT/UPDATE if specified, not on ALTER TABLE for existing rows.
                # The schema in TradingStats._create_tables() handles the CURRENT_TIMESTAMP default for new inserts.
                alter_query = f"ALTER TABLE {table_name} ADD COLUMN {column_name} {column_definition};"
                cursor.execute(alter_query)
                logger.info(f"Added missing column '{column_name}' to table '{table_name}'.")
                added_columns = True
            except sqlite3.OperationalError as e:
                # This can happen if the column definition is complex or has issues.
                # For example, adding a NOT NULL column without a default to an existing table with data.
                # Our current schema defaults to NULL or 0, which should be safe.
                logger.error(f"Failed to add column '{column_name}' to '{table_name}': {e}")
    
    if added_columns:
        logger.info(f"Schema migration: Finished adding missing columns to '{table_name}'.")
    else:
        logger.info(f"Schema migration: Table '{table_name}' is already up to date.")

def run_migrations(db_path_to_migrate: str):
    """
    Runs all database migrations on the specified database file.
    Currently, it only checks and adds missing columns to the 'trades' table.
    """
    logger.info(f"Connecting to database at {db_path_to_migrate} for migration check...")
    
    # Ensure the data directory exists before trying to connect
    data_dir = os.path.dirname(db_path_to_migrate)
    if data_dir and not os.path.exists(data_dir):
        try:
            os.makedirs(data_dir)
            logger.info(f"Created data directory for migrations: {data_dir}")
        except OSError as e:
            logger.error(f"Failed to create data directory {data_dir}: {e}")
            # If directory creation fails, we likely can't proceed with DB operations.
            return

    conn = None
    try:
        conn = sqlite3.connect(db_path_to_migrate)
        # Check and update 'trades' table
        # First, ensure the table exists (it should be created by TradingStats._create_tables if new)
        cursor = conn.cursor()
        cursor.execute("SELECT name FROM sqlite_master WHERE type='table' AND name='trades';")
        if cursor.fetchone():
            logger.info("Table 'trades' exists. Checking for missing columns...")
            add_missing_columns(conn, "trades", TRADES_TABLE_SCHEMA)
        else:
            logger.info("Table 'trades' does not exist. It will be created by TradingStats class. Skipping column check for now.")

        # Add checks for other tables here if needed in the future
        # e.g., add_missing_columns(conn, "orders", ORDERS_TABLE_SCHEMA)
        cursor.execute("SELECT name FROM sqlite_master WHERE type='table' AND name='token_stats';")
        if cursor.fetchone():
            logger.info("Table 'token_stats' exists. Checking for missing columns...")
            add_missing_columns(conn, "token_stats", TOKEN_STATS_TABLE_SCHEMA)
        else:
            logger.info("Table 'token_stats' does not exist. It will be created by TradingStats class. Skipping column check for now.")

        conn.commit()
        logger.info("Database migration check completed successfully.")
    except sqlite3.Error as e:
        logger.error(f"Database migration error: {e}")
    finally:
        if conn:
            conn.close()

if __name__ == '__main__':
    # Configure logging for direct script execution (optional)
    logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(name)s - %(message)s')
    logger.info("Running migrations directly...")
    # When running directly, use the globally defined DB_PATH
    run_migrations(DB_PATH)
    logger.info("Migrations script finished.") 