"""
Copy Trading State Management - Handles persistence and tracking of copy trading state
"""

import json
import logging
import time
import asyncio
import aiofiles
from datetime import datetime
from pathlib import Path
from typing import Dict, Set, Optional, Any
from dataclasses import dataclass, asdict
from threading import Lock

logger = logging.getLogger(__name__)


@dataclass
class CopyTradingState:
    """Represents the current state of copy trading"""
    enabled: bool = False
    target_address: Optional[str] = None
    start_timestamp: Optional[int] = None  # When copy trading was first started
    last_check_timestamp: Optional[int] = None  # Last time we checked positions
    tracked_positions: Dict[str, Dict] = None  # Positions we're already tracking
    copied_trades: Set[str] = None  # Trade IDs we've already copied
    session_start_time: Optional[int] = None  # When current session started
    
    def __post_init__(self):
        if self.tracked_positions is None:
            self.tracked_positions = {}
        if self.copied_trades is None:
            self.copied_trades = set()


class CopyTradingStateManager:
    """Manages persistence and state tracking for copy trading"""
    
    def __init__(self, state_file: str = "data/copy_trading_state.json"):
        self.state_file = Path(state_file)
        self.state = CopyTradingState()
        self._lock = Lock()
        self._initialized = False
        
        # Don't do file I/O during __init__ to prevent blocking
        # File operations will be deferred until first use
        logger.info(f"Copy trading state manager created - State file: {self.state_file}")
    
    def _ensure_initialized(self) -> None:
        """Ensure state is loaded (non-blocking, called when needed)"""
        if self._initialized:
            return
            
        try:
            # Ensure data directory exists
            self.state_file.parent.mkdir(exist_ok=True)
            
            # Load existing state
            self._load_state_sync()
            self._initialized = True
            
        except Exception as e:
            logger.error(f"Error during state initialization: {e}")
            self.state = CopyTradingState()
            self._initialized = True
    
    def _load_state_sync(self) -> None:
        """Load state from file (synchronous version for internal use)"""
        try:
            if self.state_file.exists():
                with open(self.state_file, 'r') as f:
                    data = json.load(f)
                
                # Convert copied_trades back to set
                if 'copied_trades' in data and isinstance(data['copied_trades'], list):
                    data['copied_trades'] = set(data['copied_trades'])
                
                # Update state with loaded data
                for key, value in data.items():
                    if hasattr(self.state, key):
                        setattr(self.state, key, value)
                
                logger.info(f"✅ Loaded copy trading state from {self.state_file}")
                logger.info(f"   Target: {self.state.target_address}")
                logger.info(f"   Enabled: {self.state.enabled}")
                logger.info(f"   Tracked positions: {len(self.state.tracked_positions)}")
                logger.info(f"   Copied trades: {len(self.state.copied_trades)}")
                
            else:
                logger.info(f"No existing state file found, starting fresh")
                
        except Exception as e:
            logger.error(f"Error loading copy trading state: {e}")
            logger.info("Starting with fresh state")
            self.state = CopyTradingState()
    
    async def load_state(self) -> None:
        """Load state from file (async version)"""
        try:
            if self.state_file.exists():
                async with aiofiles.open(self.state_file, 'r') as f:
                    content = await f.read()
                    data = json.loads(content)
                
                # Convert copied_trades back to set
                if 'copied_trades' in data and isinstance(data['copied_trades'], list):
                    data['copied_trades'] = set(data['copied_trades'])
                
                # Update state with loaded data
                for key, value in data.items():
                    if hasattr(self.state, key):
                        setattr(self.state, key, value)
                
                logger.info(f"✅ Loaded copy trading state from {self.state_file}")
                
            else:
                logger.info(f"No existing state file found, starting fresh")
                
        except Exception as e:
            logger.error(f"Error loading copy trading state: {e}")
            self.state = CopyTradingState()
    
    def save_state(self) -> None:
        """Save current state to file (synchronous version)"""
        try:
            with self._lock:
                # Convert state to dict
                state_dict = asdict(self.state)
                
                # Convert set to list for JSON serialization
                if 'copied_trades' in state_dict:
                    state_dict['copied_trades'] = list(state_dict['copied_trades'])
                
                # Write to temporary file first, then rename (atomic operation)
                temp_file = self.state_file.with_suffix('.tmp')
                with open(temp_file, 'w') as f:
                    json.dump(state_dict, f, indent=2)
                
                # Atomic rename
                temp_file.rename(self.state_file)
                
                logger.debug(f"💾 Saved copy trading state to {self.state_file}")
                
        except Exception as e:
            logger.error(f"Error saving copy trading state: {e}")
    
    async def save_state_async(self) -> None:
        """Save current state to file (async version)"""
        try:
            # Convert state to dict
            state_dict = asdict(self.state)
            
            # Convert set to list for JSON serialization
            if 'copied_trades' in state_dict:
                state_dict['copied_trades'] = list(state_dict['copied_trades'])
            
            # Write to temporary file first, then rename (atomic operation)
            temp_file = self.state_file.with_suffix('.tmp')
            async with aiofiles.open(temp_file, 'w') as f:
                await f.write(json.dumps(state_dict, indent=2))
            
            # Atomic rename (still sync, but very fast)
            temp_file.rename(self.state_file)
            
            logger.debug(f"💾 Saved copy trading state to {self.state_file}")
            
        except Exception as e:
            logger.error(f"Error saving copy trading state: {e}")
    
    async def start_copy_trading_async(self, target_address: str) -> None:
        """Start copy trading for a target address (async version)"""
        # Ensure state is loaded first (using async version)
        await self._ensure_initialized_async()
        
        current_time = int(time.time() * 1000)
        
        # If starting fresh or changing target, reset state
        if (not self.state.enabled or 
            self.state.target_address != target_address or
            self.state.start_timestamp is None):
            
            logger.info(f"🚀 Starting fresh copy trading session for {target_address}")
            self.state = CopyTradingState(
                enabled=True,
                target_address=target_address,
                start_timestamp=current_time,
                session_start_time=current_time,
                tracked_positions={},
                copied_trades=set()
            )
        else:
            # Resuming existing session
            logger.info(f"▶️ Resuming copy trading session for {target_address}")
            self.state.enabled = True
            self.state.session_start_time = current_time
        
        self.state.last_check_timestamp = current_time
        await self.save_state_async()
    
    async def _ensure_initialized_async(self) -> None:
        """Ensure state is loaded (async version)"""
        if self._initialized:
            return
            
        try:
            # Ensure data directory exists
            self.state_file.parent.mkdir(exist_ok=True)
            
            # Load existing state using async method
            await self.load_state()
            self._initialized = True
            
        except Exception as e:
            logger.error(f"Error during async state initialization: {e}")
            self.state = CopyTradingState()
            self._initialized = True
    
    def start_copy_trading(self, target_address: str) -> None:
        """Start copy trading for a target address"""
        # Ensure state is loaded first
        self._ensure_initialized()
        
        with self._lock:
            current_time = int(time.time() * 1000)
            
            # If starting fresh or changing target, reset state
            if (not self.state.enabled or 
                self.state.target_address != target_address or
                self.state.start_timestamp is None):
                
                logger.info(f"🚀 Starting fresh copy trading session for {target_address}")
                self.state = CopyTradingState(
                    enabled=True,
                    target_address=target_address,
                    start_timestamp=current_time,
                    session_start_time=current_time,
                    tracked_positions={},
                    copied_trades=set()
                )
            else:
                # Resuming existing session
                logger.info(f"▶️ Resuming copy trading session for {target_address}")
                self.state.enabled = True
                self.state.session_start_time = current_time
            
            self.state.last_check_timestamp = current_time
            self.save_state()
    
    def stop_copy_trading(self) -> None:
        """Stop copy trading but preserve state"""
        with self._lock:
            self.state.enabled = False
            self.state.last_check_timestamp = int(time.time() * 1000)
            self.save_state()
            logger.info("⏹️ Copy trading stopped (state preserved)")
    
    async def stop_copy_trading_async(self) -> None:
        """Stop copy trading but preserve state (async version)"""
        self.state.enabled = False
        self.state.last_check_timestamp = int(time.time() * 1000)
        await self.save_state_async()
        logger.info("⏹️ Copy trading stopped (state preserved)")
    
    def should_copy_position(self, coin: str, position_data: Dict) -> bool:
        """Determine if we should copy a position"""
        self._ensure_initialized()
        with self._lock:
            # If we haven't seen this position before, it's new
            if coin not in self.state.tracked_positions:
                return True
            
            # If we've seen it but it's larger than before, copy the increase
            previous_size = self.state.tracked_positions[coin].get('size', 0)
            current_size = position_data.get('size', 0)
            
            return current_size > previous_size
    
    def is_position_reduction(self, coin: str, position_data: Dict) -> bool:
        """Check if this is a position reduction"""
        with self._lock:
            if coin not in self.state.tracked_positions:
                return False
            
            previous_size = self.state.tracked_positions[coin].get('size', 0)
            current_size = position_data.get('size', 0)
            
            return current_size < previous_size
    
    def is_position_closure(self, coin: str) -> bool:
        """Check if a tracked position has been closed"""
        with self._lock:
            return coin in self.state.tracked_positions
    
    def update_tracked_position(self, coin: str, position_data: Dict) -> None:
        """Update our tracking of a position"""
        with self._lock:
            self.state.tracked_positions[coin] = {
                'size': position_data.get('size', 0),
                'side': position_data.get('side'),
                'entry_price': position_data.get('entry_price', 0),
                'leverage': position_data.get('leverage', 1),
                'last_updated': int(time.time() * 1000)
            }
            self.save_state()
    
    async def update_tracked_position_async(self, coin: str, position_data: Dict) -> None:
        """Update our tracking of a position (async version)"""
        self.state.tracked_positions[coin] = {
            'size': position_data.get('size', 0),
            'side': position_data.get('side'),
            'entry_price': position_data.get('entry_price', 0),
            'leverage': position_data.get('leverage', 1),
            'last_updated': int(time.time() * 1000)
        }
        await self.save_state_async()
    
    def remove_tracked_position(self, coin: str) -> None:
        """Remove a position from tracking (when closed)"""
        with self._lock:
            if coin in self.state.tracked_positions:
                del self.state.tracked_positions[coin]
                self.save_state()
                logger.info(f"🗑️ Removed {coin} from tracked positions")
    
    async def remove_tracked_position_async(self, coin: str) -> None:
        """Remove a position from tracking (when closed) (async version)"""
        if coin in self.state.tracked_positions:
            del self.state.tracked_positions[coin]
            await self.save_state_async()
            logger.info(f"🗑️ Removed {coin} from tracked positions")
    
    def add_copied_trade(self, trade_id: str) -> None:
        """Mark a trade as copied"""
        with self._lock:
            self.state.copied_trades.add(trade_id)
            # Keep only last 1000 trade IDs to prevent unbounded growth
            if len(self.state.copied_trades) > 1000:
                oldest_trades = sorted(self.state.copied_trades)[:500]
                for trade_id in oldest_trades:
                    self.state.copied_trades.discard(trade_id)
            self.save_state()
    
    async def add_copied_trade_async(self, trade_id: str) -> None:
        """Mark a trade as copied (async version)"""
        self.state.copied_trades.add(trade_id)
        # Keep only last 1000 trade IDs to prevent unbounded growth
        if len(self.state.copied_trades) > 1000:
            oldest_trades = sorted(self.state.copied_trades)[:500]
            for trade_id in oldest_trades:
                self.state.copied_trades.discard(trade_id)
        await self.save_state_async()
    
    def has_copied_trade(self, trade_id: str) -> bool:
        """Check if we've already copied a trade"""
        self._ensure_initialized()
        with self._lock:
            return trade_id in self.state.copied_trades
    
    def update_last_check(self) -> None:
        """Update the last check timestamp"""
        with self._lock:
            self.state.last_check_timestamp = int(time.time() * 1000)
            self.save_state()
    
    async def update_last_check_async(self) -> None:
        """Update the last check timestamp (async version)"""
        self.state.last_check_timestamp = int(time.time() * 1000)
        await self.save_state_async()
    
    def initialize_tracked_positions(self, current_positions: Dict) -> None:
        """Initialize tracking with current target positions (on first start)"""
        with self._lock:
            logger.info(f"🔄 Initializing position tracking with {len(current_positions)} existing positions")
            
            for coin, position in current_positions.items():
                self.state.tracked_positions[coin] = {
                    'size': position.size,
                    'side': position.side,
                    'entry_price': position.entry_price,
                    'leverage': position.leverage,
                    'last_updated': int(time.time() * 1000)
                }
                logger.info(f"   📊 Tracking existing {position.side.upper()} {coin}: {position.size}")
            
            self.save_state()
    
    async def initialize_tracked_positions_async(self, current_positions: Dict) -> None:
        """Initialize tracking with current target positions (on first start) (async version)"""
        logger.info(f"🔄 Initializing position tracking with {len(current_positions)} existing positions")
        
        for coin, position in current_positions.items():
            self.state.tracked_positions[coin] = {
                'size': position.size,
                'side': position.side,
                'entry_price': position.entry_price,
                'leverage': position.leverage,
                'last_updated': int(time.time() * 1000)
            }
            logger.info(f"   📊 Tracking existing {position.side.upper()} {coin}: {position.size}")
        
        await self.save_state_async()
    
    def get_session_info(self) -> Dict[str, Any]:
        """Get information about current session"""
        with self._lock:
            start_time = None
            session_duration = None
            
            if self.state.start_timestamp:
                start_time = datetime.fromtimestamp(self.state.start_timestamp / 1000)
            
            if self.state.session_start_time:
                session_duration = (int(time.time() * 1000) - self.state.session_start_time) / 1000
            
            return {
                'enabled': self.state.enabled,
                'target_address': self.state.target_address,
                'start_time': start_time,
                'session_duration_seconds': session_duration,
                'tracked_positions_count': len(self.state.tracked_positions),
                'copied_trades_count': len(self.state.copied_trades),
                'last_check_time': datetime.fromtimestamp(self.state.last_check_timestamp / 1000) if self.state.last_check_timestamp else None
            }
    
    def reset_state(self) -> None:
        """Reset all state (useful for testing or manual reset)"""
        with self._lock:
            self.state = CopyTradingState()
            self.save_state()
            logger.info("🔄 Copy trading state reset")
    
    def get_tracked_positions(self) -> Dict[str, Dict]:
        """Get currently tracked positions"""
        self._ensure_initialized()
        with self._lock:
            return self.state.tracked_positions.copy()
    
    def is_enabled(self) -> bool:
        """Check if copy trading is enabled"""
        self._ensure_initialized()
        with self._lock:
            return self.state.enabled
    
    def get_target_address(self) -> Optional[str]:
        """Get the target address"""
        self._ensure_initialized()
        with self._lock:
            return self.state.target_address 