#!/usr/bin/env python3
"""
Exchange Order Synchronization Manager
Syncs orders from the exchange to the database for tracking
"""

import logging
from typing import Dict, List, Optional, Any
from datetime import datetime, timezone
import uuid

logger = logging.getLogger(__name__)

class ExchangeOrderSync:
    """Manages synchronization of exchange orders with database"""
    
    def __init__(self, hl_client, trading_stats):
        self.hl_client = hl_client
        self.trading_stats = trading_stats
        
    def sync_exchange_orders_to_database(self) -> Dict[str, Any]:
        """Sync all exchange orders to database for tracking"""
        try:
            # Get current exchange orders
            exchange_orders = self.hl_client.get_open_orders() or []
            
            # Get current database orders
            db_orders = []
            db_orders.extend(self.trading_stats.get_orders_by_status('open', limit=100))
            db_orders.extend(self.trading_stats.get_orders_by_status('submitted', limit=100))
            db_orders.extend(self.trading_stats.get_orders_by_status('pending_trigger', limit=100))
            
            # Track what we sync
            sync_results = {
                'new_orders_added': 0,
                'orders_updated': 0,
                'orphaned_orders_cancelled': 0,
                'errors': []
            }
            
            # Sync exchange orders to database
            self._sync_exchange_to_db(exchange_orders, db_orders, sync_results)
            
            # Clean up orphaned database orders
            self._cleanup_orphaned_db_orders(exchange_orders, db_orders, sync_results)
            
            logger.info(f"Order sync complete: {sync_results['new_orders_added']} added, "
                       f"{sync_results['orders_updated']} updated, "
                       f"{sync_results['orphaned_orders_cancelled']} orphaned")
            
            return sync_results
            
        except Exception as e:
            logger.error(f"Error syncing exchange orders: {e}")
            return {'error': str(e)}
    
    def _sync_exchange_to_db(self, exchange_orders: List[Dict], db_orders: List[Dict], sync_results: Dict):
        """Sync exchange orders to database"""
        
        # Create lookup of existing database orders by exchange ID
        db_orders_by_exchange_id = {
            order.get('exchange_order_id'): order 
            for order in db_orders 
            if order.get('exchange_order_id')
        }
        
        for exchange_order in exchange_orders:
            try:
                exchange_order_id = exchange_order.get('id')
                if not exchange_order_id:
                    continue
                
                # Check if order already exists in database
                if exchange_order_id in db_orders_by_exchange_id:
                    # Update existing order
                    self._update_existing_order(exchange_order, db_orders_by_exchange_id[exchange_order_id], sync_results)
                else:
                    # Add new order to database
                    self._add_new_order_to_database(exchange_order, sync_results)
                    
            except Exception as e:
                sync_results['errors'].append(f"Error syncing order {exchange_order.get('id', 'unknown')}: {e}")
                logger.error(f"Error syncing exchange order: {e}")
    
    def _add_new_order_to_database(self, exchange_order: Dict, sync_results: Dict):
        """Add a new exchange order to the database"""
        try:
            symbol = exchange_order.get('symbol', '')
            side = exchange_order.get('side', '')
            order_type = exchange_order.get('type', '')
            amount = float(exchange_order.get('amount', 0))
            price = float(exchange_order.get('price', 0)) if exchange_order.get('price') else None
            exchange_order_id = exchange_order.get('id', '')
            
            # Generate bot order ref ID for tracking
            bot_order_ref_id = f"sync_{uuid.uuid4().hex[:8]}"
            
            # Determine order status
            status = self._map_exchange_status_to_db_status(exchange_order)
            
            # Check if it's a reduce-only order
            is_reduce_only = exchange_order.get('reduceOnly', False) or exchange_order.get('info', {}).get('reduceOnly', False)
            
            # Skip zero-size orders (position placeholders)
            if amount == 0:
                logger.debug(f"Skipping zero-size order {exchange_order_id} (position placeholder)")
                return
            
            # Insert into database
            insert_query = """
                INSERT INTO orders (
                    bot_order_ref_id, exchange_order_id, symbol, side, type, 
                    amount_requested, amount_filled, price, status, 
                    timestamp_created, timestamp_updated
                ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
            """
            
            timestamp = datetime.now(timezone.utc).isoformat()
            amount_filled = float(exchange_order.get('filled', 0))
            
            params = (
                bot_order_ref_id, exchange_order_id, symbol, side, order_type,
                amount, amount_filled, price, status,
                timestamp, timestamp
            )
            
            self.trading_stats.db_manager._execute_query(insert_query, params)
            sync_results['new_orders_added'] += 1
            
            logger.info(f"✅ Added exchange order to database: {symbol} {side} {order_type} (ID: {exchange_order_id})")
            
        except Exception as e:
            logger.error(f"Error adding exchange order to database: {e}")
            sync_results['errors'].append(f"Error adding order {exchange_order.get('id', 'unknown')}: {e}")
    
    def _update_existing_order(self, exchange_order: Dict, db_order: Dict, sync_results: Dict):
        """Update existing database order with exchange data"""
        try:
            exchange_order_id = exchange_order.get('id', '')
            new_status = self._map_exchange_status_to_db_status(exchange_order)
            new_amount_filled = float(exchange_order.get('filled', 0))
            
            # Check if status or filled amount changed
            old_status = db_order.get('status', '')
            old_amount_filled = float(db_order.get('amount_filled', 0))
            
            if new_status != old_status or abs(new_amount_filled - old_amount_filled) > 0.0001:
                # Update the order
                update_query = """
                    UPDATE orders 
                    SET status = ?, amount_filled = ?, timestamp_updated = ?
                    WHERE exchange_order_id = ?
                """
                
                timestamp = datetime.now(timezone.utc).isoformat()
                params = (new_status, new_amount_filled, timestamp, exchange_order_id)
                
                self.trading_stats.db_manager._execute_query(update_query, params)
                sync_results['orders_updated'] += 1
                
                logger.debug(f"Updated order {exchange_order_id}: status {old_status} -> {new_status}, "
                           f"filled {old_amount_filled} -> {new_amount_filled}")
            
        except Exception as e:
            logger.error(f"Error updating exchange order in database: {e}")
            sync_results['errors'].append(f"Error updating order {exchange_order.get('id', 'unknown')}: {e}")
    
    def _cleanup_orphaned_db_orders(self, exchange_orders: List[Dict], db_orders: List[Dict], sync_results: Dict):
        """Clean up database orders that no longer exist on exchange"""
        
        # Get exchange order IDs
        exchange_order_ids = {order.get('id') for order in exchange_orders if order.get('id')}
        
        for db_order in db_orders:
            try:
                exchange_order_id = db_order.get('exchange_order_id')
                if not exchange_order_id:
                    continue  # Skip orders without exchange ID
                
                # If database order not found on exchange, mark as cancelled
                if exchange_order_id not in exchange_order_ids:
                    self._mark_order_as_cancelled(db_order, sync_results)
                    
            except Exception as e:
                logger.error(f"Error checking orphaned order: {e}")
                sync_results['errors'].append(f"Error checking orphaned order: {e}")
    
    def _mark_order_as_cancelled(self, db_order: Dict, sync_results: Dict):
        """Mark a database order as cancelled (no longer on exchange)"""
        try:
            order_id = db_order.get('id')
            exchange_order_id = db_order.get('exchange_order_id', '')
            symbol = db_order.get('symbol', '')
            
            update_query = """
                UPDATE orders 
                SET status = 'cancelled', timestamp_updated = ?
                WHERE id = ?
            """
            
            timestamp = datetime.now(timezone.utc).isoformat()
            params = (timestamp, order_id)
            
            self.trading_stats.db_manager._execute_query(update_query, params)
            sync_results['orphaned_orders_cancelled'] += 1
            
            logger.info(f"🗑️ Marked orphaned order as cancelled: {symbol} (Exchange ID: {exchange_order_id})")
            
        except Exception as e:
            logger.error(f"Error marking order as cancelled: {e}")
    
    def _map_exchange_status_to_db_status(self, exchange_order: Dict) -> str:
        """Map exchange order status to database status"""
        exchange_status = exchange_order.get('status', '').lower()
        
        status_mapping = {
            'open': 'open',
            'pending': 'submitted',
            'closed': 'filled',
            'canceled': 'cancelled',
            'cancelled': 'cancelled',
            'expired': 'expired',
            'rejected': 'failed'
        }
        
        return status_mapping.get(exchange_status, 'unknown') 