import asyncio
import logging
from typing import Dict, List, Optional, Any

from ..clients.hyperliquid_client import HyperliquidClient
from ..notifications.notification_manager import NotificationManager
from ..config.config import Config

logger = logging.getLogger(__name__)

class RiskManager:
    """
    Simplified risk manager that monitors for ROE hard exits.
    Closes positions when they hit the configured ROE threshold.
    """
    
    def __init__(self, hl_client: HyperliquidClient, notification_manager: NotificationManager, config: Config):
        self.hl_client = hl_client
        self.notification_manager = notification_manager
        self.config = config
        self.is_running = False
        
        # Risk thresholds from config (convert percentage to decimal)
        # Note: Exchange ROE is positive for profits, negative for losses
        self.hard_exit_roe = -(config.STOP_LOSS_PERCENTAGE / 100.0)  # 10.0% -> -0.10 ROE (loss threshold)
        
    async def start(self):
        """Start risk manager"""
        if self.is_running:
            return
            
        self.is_running = True
        logger.info(f"Starting risk manager with hard exit ROE: {self.hard_exit_roe}")
        
        # Start monitoring loop
        asyncio.create_task(self._monitoring_loop())
        
    async def stop(self):
        """Stop risk manager"""
        self.is_running = False
        logger.info("Stopping risk manager")
        
    async def _monitoring_loop(self):
        """Main monitoring loop"""
        while self.is_running:
            try:
                await self._check_risk_thresholds()
                await asyncio.sleep(self.config.BOT_HEARTBEAT_SECONDS)  # Use config heartbeat
            except Exception as e:
                logger.error(f"Error in risk manager monitoring loop: {e}")
                await asyncio.sleep(self.config.BOT_HEARTBEAT_SECONDS)
                
    async def _check_risk_thresholds(self):
        """Check if any positions need risk management"""
        try:
            positions = self.hl_client.get_positions()
            if not positions:
                return
                
            positions_to_close = []
            
            for position in positions:
                # Access nested position data from info.position
                position_data = position.get('info', {}).get('position', {})
                if not position_data:
                    continue
                    
                size = float(position_data.get('szi', '0'))
                if size != 0:
                    symbol = position_data.get('coin', '')
                    roe = float(position_data.get('returnOnEquity', '0'))
                    
                    # Check if ROE exceeds hard exit threshold (negative ROE = loss)
                    if roe <= self.hard_exit_roe:
                        positions_to_close.append({
                            'symbol': symbol,
                            'size': size,
                            'roe': roe,
                            'unrealized_pnl': float(position_data.get('unrealizedPnl', '0'))
                        })
                        
            # Close positions that exceed risk threshold
            for position in positions_to_close:
                await self._execute_hard_exit(position)
                
        except Exception as e:
            logger.error(f"Error checking risk thresholds: {e}")
            
    async def _execute_hard_exit(self, position: Dict):
        """Execute hard exit for a position"""
        try:
            symbol = position['symbol']
            size = position['size']
            roe = position['roe']
            unrealized_pnl = position['unrealized_pnl']
            
            # Determine order side (opposite of position)
            side = 'sell' if size > 0 else 'buy'
            order_size = abs(size)
            
            logger.warning(f"Hard exit triggered for {symbol}: ROE {roe:.2%} < {self.hard_exit_roe:.2%}")
            
            # Place market order to close position
            full_symbol = f"{symbol}/USDC:USDC"
            order_result, error_msg = self.hl_client.place_market_order(
                symbol=full_symbol,
                side=side,
                amount=order_size
            )
            
            if order_result and not error_msg:
                # Send alert notification
                message = (
                    f"🚨 HARD EXIT EXECUTED\n"
                    f"Token: {symbol}\n"
                    f"ROE: {roe:.2%}\n"
                    f"Threshold: {self.hard_exit_roe:.2%}\n"
                    f"Size: {order_size:.4f}\n"
                    f"Est. PnL: ${unrealized_pnl:.3f}\n"
                    f"Risk management action taken!"
                )
                
                await self.notification_manager.send_generic_notification(message)
                logger.info(f"Hard exit executed for {symbol}: {order_size} @ ROE {roe:.2%}")
                
            else:
                logger.error(f"Failed to execute hard exit for {symbol}: {error_msg}")
                
                # Send failure notification
                error_message = (
                    f"❌ HARD EXIT FAILED\n"
                    f"Token: {symbol}\n"
                    f"ROE: {roe:.2%}\n"
                    f"Could not place market order!\n"
                    f"Error: {error_msg}\n"
                    f"Manual intervention required!"
                )
                await self.notification_manager.send_generic_notification(error_message)
                
        except Exception as e:
            logger.error(f"Error executing hard exit for {position['symbol']}: {e}")
            
            # Send error notification
            error_message = (
                f"❌ HARD EXIT ERROR\n"
                f"Token: {position['symbol']}\n"
                f"ROE: {position['roe']:.2%}\n"
                f"Error: {str(e)}\n"
                f"Manual intervention required!"
            )
            await self.notification_manager.send_generic_notification(error_message)
            
    async def get_risk_status(self) -> Dict:
        """Get current risk status"""
        try:
            positions_data = self.hl_client.get_positions()
            if not positions_data:
                return {'positions': [], 'total_risk': 0}
                
            positions = []
            total_unrealized_pnl = 0
            
            for position in positions_data:
                # Access nested position data from info.position
                position_data = position.get('info', {}).get('position', {})
                if not position_data:
                    continue
                    
                size = float(position_data.get('szi', '0'))
                if size != 0:
                    symbol = position_data.get('coin', '')
                    roe = float(position_data.get('returnOnEquity', '0'))
                    unrealized_pnl = float(position_data.get('unrealizedPnl', '0'))
                    
                    total_unrealized_pnl += unrealized_pnl
                    
                    # Calculate risk level
                    risk_level = "HIGH" if roe <= self.hard_exit_roe * 0.8 else "MEDIUM" if roe <= self.hard_exit_roe * 0.5 else "LOW"
                    
                    positions.append({
                        'symbol': symbol,
                        'size': size,
                        'roe': roe,
                        'unrealized_pnl': unrealized_pnl,
                        'risk_level': risk_level,
                        'distance_to_hard_exit': roe - self.hard_exit_roe
                    })
                    
            return {
                'positions': positions,
                'total_unrealized_pnl': total_unrealized_pnl,
                'hard_exit_threshold': self.hard_exit_roe,
                'high_risk_positions': len([p for p in positions if p['risk_level'] == 'HIGH'])
            }
            
        except Exception as e:
            logger.error(f"Error getting risk status: {e}")
            return {'positions': [], 'total_risk': 0} 